#!/usr/bin/env python

# Visual python script to animate a molecular dynamics trajectory in xyz format.
# Author: Daniele Coslovich (daniele.coslovich@umontpellier.fr)

import sys
import visual

_particle = []

# Reduce this value to make animation faster
visual.rate(10)

def clear():
    """Clear already drawn particles"""
    for p in _particle:
        p.visible = False
        del p

def box(side):    
    """Draw a simulation cell of side L"""
    L = side[0] / 2
    visual.curve(pos=[(-L,-L,-L), (-L,-L,L), (L,-L,L), (L,-L,-L), (-L,-L,-L)], color=visual.color.white, radius=.02)
    visual.curve(pos=[(-L, L,-L), (-L, L,L), (L, L,L), (L, L,-L), (-L, L,-L)], color=visual.color.white, radius=.02)
    visual.curve(pos=[(-L,-L,-L), (-L,-L,L), (-L,L,L), (-L,L,-L), (-L,-L,-L)], color=visual.color.white, radius=.02)
    visual.curve(pos=[( L,-L,-L), ( L,-L,L), ( L,L,L), ( L,L,-L), ( L,-L,-L)], color=visual.color.white, radius=.02)

def show(pos, radius=None, color=None, append=False):
    """Display particles at positions pos. 
    The optional lists radius and color allow to customize particles' appearance.
    """
    if radius is None:
        radius = [0.2] * len(pos)
    if color is None:
        color = [visual.color.white] * len(pos)

    if len(_particle) == 0 or append:
        for x, r, c in zip(pos, radius, color):
            _particle.append(visual.sphere(pos=x, radius=r, color=c))
    else:
        for x, r, c, p in zip(pos, radius, color, _particle):
            p.pos = x
            p.radius = r
            p.col= c
            p.visible = True

def read(f):
    """Read the whole configuration from file f in xyz format.
    Return: a list of lists of box sides, positions and velocities."""
    pos_all = []
    vel_all = []
    box_all = []
    with open(f) as fh:
        data = fh.readlines()
        npart = int(data[0])
        # Assume number of particles is constant, it can be relaxed
        nconf = len(data) / (npart+2)
        for iconf in range(nconf):
            off = iconf*(npart+2)
            off_next = (iconf+1)*(npart+2)
            box = map(float, data[off+1].split())
            pos = []
            vel = []
            for line in data[off+2:off_next]:
                line_data = line.split()
                pos.append(map(float, line_data[1:4]))
                vel.append(map(float, line_data[4:7]))
            box_all.append(box)
            pos_all.append(pos)
            vel_all.append(vel)
    return box_all, pos_all, vel_all

def main(f):
    """Display an animation of the trajectory contained in file f."""
    L, pos, vel = read(f)
    box(L[0])
    for p in pos:
        show(p)

if __name__ == '__main__':
    if len(sys.argv) == 1 or sys.argv[1] == '-h':
        print "Usage: %s <file>" % sys.argv[0]
        print "where <file> is the trajectory file to visualize"
        sys.exit()

    main(sys.argv[1])
    
